@@ -1,15 +1,18 @@
-import os
-import stat
-import subprocess
+"""
+Library for all things related to running a Harmony node with AutoNode.
+"""
+
 import glob
 import json
-import time
-import shutil
 import logging
-
-import requests
+import os
+import shutil
+import stat
+import subprocess
+import time
 
 import pyhmy.rpc.exceptions as rpc_exception
+import requests
 from pyhmy import (
     blockchain,
     cli,
@@ -25,10 +28,10 @@ from .common import (
     bls_key_dir,
     harmony_dir
 )
-
 from .util import (
     input_with_print,
-    get_simple_rotating_log_handler
+    get_simple_rotating_log_handler,
+    is_bls_file,
 )
 
 node_sh_out_path = f"{node_sh_log_dir}/out.log"
@@ -36,7 +39,7 @@ node_sh_err_path = f"{node_sh_log_dir}/err.log"
 
 log_path = f"{harmony_dir}/autonode_node.log"
 
-rclone_space_buffer = 5 * 2**30  # 5GB in bytes
+rclone_space_buffer = 5 * 2 ** 30  # 5GB in bytes
 rclone_config = "harmony"
 
 
@@ -169,7 +172,8 @@ def _get_node_shard():
             log(f'{Typgpy.WARNING}[!] Failed to get shard for bls key {bls_key}!{Typgpy.ENDC}')
     if not key_shards:
         return None
-    assert len(set(key_shards)) == 1, f"Node BLS keys can only be for 1 shard. BLS keys: {node_config['public-bls-keys']}"
+    assert len(
+        set(key_shards)) == 1, f"Node BLS keys can only be for 1 shard. BLS keys: {node_config['public-bls-keys']}"
     return key_shards[0]
 
 
@@ -189,6 +193,7 @@ def start(auto=False, verbose=True):
             f"this is not recommended, continue? [Y]/n{Typgpy.ENDC}")
         if input_with_print("> ") not in {'Y', 'y', 'yes', 'Yes'}:
             raise SystemExit()
+    assert_valid_bls_key_directory()
     os.chdir(node_dir)
     if os.path.isfile(f"{node_dir}/node.sh"):
         os.remove(f"{node_dir}/node.sh")
@@ -204,7 +209,7 @@ def start(auto=False, verbose=True):
     st = os.stat("node.sh")
     os.chmod("node.sh", st.st_mode | stat.S_IEXEC)
     node_args = ["./node.sh", "-N", node_config["network"], "-z", "-f", bls_key_dir, "-M", "-S"]
-    if node_config['clean'] and node_config['network'] != 'mainnet':
+    if node_config['clean']:
         if verbose:
             log(f"{Typgpy.WARNING}[!] Cleaning up old files before starting node.{Typgpy.ENDC}")
         _node_clean(verbose=verbose)
@@ -268,7 +273,8 @@ def assert_no_invalid_blocks():
         files = glob.glob(f"{node_dir}/latest/zero*.log")
         if files:
             log_path = files[-1]
-            assert not has_invalid_block(log_path), f"`invalid merkle root` present in {log_path}, restart AutoNode with clean option"
+            assert not has_invalid_block(
+                log_path), f"`invalid merkle root` present in {log_path}, restart AutoNode with clean option"
 
 
 def is_signing(count=1500):
@@ -279,7 +285,8 @@ def is_signing(count=1500):
         files = glob.glob(f"{node_dir}/latest/zero*.log")
         if files:
             log_path = files[-1]
-            content = subprocess.check_output(["tail", "-n", str(count), str(log_path)], env=os.environ).decode().split("\n")
+            content = subprocess.check_output(["tail", "-n", str(count), str(log_path)], env=os.environ).decode().split(
+                "\n")
             for line in content:
                 line = line.rstrip()
                 if "BINGO" in line or "HOORAY" in line:
@@ -300,6 +307,36 @@ def has_invalid_block(log_file_path):
     return False
 
 
+def assert_valid_bls_key_directory():
+    """
+    Asserts that the BLS keys directory contains the BLS keys for the
+    given node config.
+
+    Note that this must match EXACTLY.
+
+    Will raise assertion error if one BLS key is missing OR
+    if there is 1 BLS key more than what is in the node config.
+    """
+    bls_keys = list(filter(lambda e: is_bls_file(e, '.key'), os.listdir(bls_key_dir)))
+    bls_pass = list(filter(lambda e: is_bls_file(e, '.pass'), os.listdir(bls_key_dir)))
+
+    assert len(bls_keys) == len(bls_pass), f"number of BLS keys and BLS pass files must be equal in {bls_key_dir}."
+    bls_pub_keys = set(x.replace('.key', '') for x in bls_keys)
+    assert len(bls_pub_keys) == len(bls_keys), f"sanity check: cannot contain duplicate BLS keys in {bls_key_dir}"
+    found_keys_count = len(bls_pub_keys)
+
+    for key in bls_pass:
+        bls_key = key.replace('.pass', '')
+        assert bls_key in bls_pub_keys, f"got BLS pass for {bls_key}, but BLS key not found in {bls_key_dir}"
+
+    conf_keys_count = len(node_config['public-bls-keys'])
+    if conf_keys_count != found_keys_count:
+        raise AssertionError(f"number of configured BLS keys ({conf_keys_count}) not equal to number of BLS keys "
+                             f"keys ({found_keys_count}) in {bls_key_dir}")
+    for key in node_config['public-bls-keys']:
+        assert key in bls_pub_keys, f"configured BLS key {key} not found in {bls_key_dir}"
+
+
 def assert_started(timeout=60, do_log=False):
     """
     Assert the node has started within the given timeout.
