@@ -1,29 +1,36 @@
+"""
+Library of common utils used by most function libraries of AutoNode.
+
+To prevent cyclic import minimize the importing of other libraries in AutoNode.
+"""
+
 import gzip
-import os
 import logging
 import logging.handlers
+import os
 import sys
 
 import pexpect
-from pyhmy import cli
 from pyhmy import (
     Typgpy,
     json_load
 )
+from pyhmy import cli
 
 from .common import (
     log,
     node_config,
     validator_config,
-    msg_tag
+    msg_tag,
+    bls_key_len
+)
+from .exceptions import (
+    InvalidWalletPassphrase
 )
 from .passphrase import (
     decrypt_wallet_passphrase,
     is_valid_passphrase
 )
-from .exceptions import (
-    InvalidWalletPassphrase
-)
 
 
 class _GZipRotator:
@@ -111,3 +118,12 @@ def shard_for_bls(public_bls_key):
     """
     return json_load(cli.single_call(['hmy', '--node', f'{node_config["endpoint"]}', 'utility',
                                       'shard-for-bls', public_bls_key]))['shard-id']
+
+
+def is_bls_file(file_name, suffix):
+    if file_name.startswith('.') or not file_name.endswith(suffix):
+        return False
+    tok = file_name.split(".")
+    if len(tok) != 2 or len(tok[0]) != bls_key_len:
+        return False
+    return True
